import scala.io._

object LJBFSExp extends App {

  val gFile = args(0)
  val g = Graph(gFile)
  val qFile = "ljqsmallbfs_"+gFile+".txt"
  println("LJ size: " + g.size)
  println("LJ edges: " + g.nEdges)

  val smallQueries = Array((2, 1), (4, 2), (6, 2), (8, 2), (10, 2))
 
  // BFS queries
  for (i <- 0 until 100) {
    for ((s, deg) <- smallQueries) {
      val name = s + "_" + deg + "_bfs_" + i
      val q = GraphGenerator.generateBFSQuery(s, deg, g)
      q.writeToFile(qFile)
      Utils.runAllMyExps(g, q, gFile, qFile, name)
    }
  }
}


object LJBigBFSExp extends App {

  val gFile = args(0)
  val g = Graph(gFile) 
  val qFile = "ljqbigbfs_"+gFile+".txt"

  val bigQueries = Array((6, 2), (8, 2), (10, 2))
 
  // BFS queries
  for (i <- 0 until 100) {
    for ((s, deg) <- bigQueries) {
      val name = s + "_" + deg + "_bfs_" + i
      val q = GraphGenerator.generateBFSQuery(s, deg, g)
      q.writeToFile(qFile)
      Utils.runAllFastExps(g, q, gFile, qFile, name)
    }
  }
}


object LJRandExp extends App {
  val gFile = args(0)
  val g = Graph(gFile) 
  val qFile = "ljqsmallrand_"+gFile+".txt"

  val smallQueries = Array((2, 1), (4, 2), (6, 2), (8, 2), (10, 2))

  // Rand queries
  for (i <- 0 until 100) {
    for ((s, deg) <- smallQueries) {
      val name = s + "_" + deg + "_rand_" + i
      val q = GraphGenerator.generateRandomGraph(s, g.nLabels, deg) 
      q.writeToFile(qFile)
      println("Done generating query graph")
      Utils.runAllMyExps(g, q, gFile, qFile, name)
    }
  }
}


object LJBigRandExp extends App {
  val gFile = args(0)
  val g = Graph(gFile)
  val qFile = "ljqbigrand_"+gFile+".txt"

  val bigQueries = Array((6, 2), (8, 2), (10, 2))

  // Rand queries
  for (i <- 0 until 100) {
    for ((s, deg) <- bigQueries) {
      val name = s + "_" + deg + "_rand_" + i
      val q = GraphGenerator.generateRandomGraph(s, g.nLabels, deg) 
      q.writeToFile(qFile)
      Utils.runAllFastExps(g, q, gFile, qFile, name)
    }
  }
}

object LJRandMaker extends App{
  val file = args(0) 
  val outFile = args(1)
  var g = Graph(file)
  val l = 100
  val randLabels = GraphGenerator.randDistLabels(g.size, l)
  g = new Graph(g.adjList, randLabels, 
                Graph.buildLabelMapFromLabels(randLabels))
  g.writeToFile(outFile)
}
object LJMaker extends App{
  val file = args(0) 
  val outFile = args(1)
  var g = Graph(file)
  val l = 100
  val power = 2.1
  val powLabels = GraphGenerator.powDistLabels(g.size, l, power)
  g = new Graph(g.adjList, powLabels, 
                Graph.buildLabelMapFromLabels(powLabels))
  g.writeToFile(outFile)
}

object LJLabelDist extends App {
  var g = Graph(args(0))
  g.labelMap.foreach{ case (x, y) => println(x + " -> " + y.size) }
  //g.labelMap.values.toArray.sortBy(-1 * _).foreach(println(_))
}
