import java.io._
import scala.collection.{mutable => mut}
import scala.actors._

object PatternMatcher {

  def ullmann(g: Graph, q: Graph): Set[Array[Set[Int]]] = {
    val start = System.nanoTime()
    var matches = Set[Array[Set[Int]]]()
    var initPhi = saltzGraphSim(g, q, feasibleMates2(g, q))
    ullmann_(g, q, initPhi, 
              q.adjList.zipWithIndex.sortBy(-1 * _._1.size).map(_._2), 
              0) 
  
    def ullmann_(g: Graph, q: Graph, phi: Array[Set[Int]], 
                  ordering: Array[Int], depth: Int) {
      if (depth == q.size) { 
        if (!phi.isEmpty){
          matches += phi
          if (matches.size == 1000)
            println("1000 ullmann: " + (System.nanoTime()-start)/1000000.0)
          if (matches.size % 5000000 == 0) 
            println("matches so far: " + matches.size)
        }
      }
      else if (!phi.isEmpty) {
        for (i <- phi(ordering(depth)) if (!contains(phi, i, ordering, depth))) {
          val phiCopy = phi.map(x=>x)
          phiCopy(ordering(depth)) = Set[Int](i)
//          if (matches.size >= 10000) return
          ullmann_(g, q, saltzGraphSim(g, q, phiCopy), ordering, depth + 1)
        }
      }
    }

    def contains(phi: Array[Set[Int]], ele: Int, ordering: Array[Int], 
                 depth: Int): Boolean = {
      for (i <- 0 until depth) if (phi(ordering(i)) contains ele) return true
      false 
    }
    matches 
  }

  def myUllmann(g: Graph, q: Graph): Set[Array[Set[Int]]] = {
    val start = System.nanoTime()
    var matches = Set[Array[Set[Int]]]()
    var initPhi = saltzGraphSim(g, q, feasibleMates(g, q))
    ullmann_(g, q, initPhi, 
              q.adjList.zipWithIndex.sortBy(-1 * _._1.size).map(_._2), 
              0) 
  
    def ullmann_(g: Graph, q: Graph, phi: Array[Set[Int]], 
                  ordering: Array[Int], depth: Int) {
      if (depth == q.size) { 
        if (!phi.isEmpty){
          matches += phi
          if (matches.size == 1000)
            println("1000 myUllmann: " + (System.nanoTime()-start)/1000000.0)
          if (matches.size % 5000000 == 0) 
            println("matches so far: " + matches.size)
        }
      }
      else if (!phi.isEmpty) {
        for (i <- phi(ordering(depth)) if (!contains(phi, i, ordering, depth))) {
          val phiCopy = phi.map(x=>x)
          phiCopy(ordering(depth)) = Set[Int](i)
//          if (matches.size >= 10000) return
          ullmann_(g, q, saltzGraphSim(g, q, phiCopy), ordering, depth + 1)
        }
      }
    }

    def contains(phi: Array[Set[Int]], ele: Int, ordering: Array[Int], 
                 depth: Int): Boolean = {
      for (i <- 0 until depth) if (phi(ordering(i)) contains ele) return true
      false 
    }
    matches 
  }

  def saltzIsoAltFeas(g: Graph, q: Graph): Set[Array[Set[Int]]] = {
    val start = System.nanoTime()
    var matches = Set[Array[Set[Int]]]()
    saltzIso_(g, q, saltzDualSim(g, q, feasibleMates2(g, q)), 0) 
  
    def saltzIso_(g: Graph, q: Graph, phi: Array[Set[Int]], depth: Int) {
      if (depth == q.size) {
        if (!phi.isEmpty) { 
          matches += phi
        if (matches.size == 1000)
          println("1000 saltzIsoAltFeas: " + (System.nanoTime()-start)/1000000.0)
        if (matches.size % 5000000 == 0)
          println("matches so far: " + matches.size)
        }
      }
      else if (!phi.isEmpty) {
        for (i <- phi(depth) if (!contains(phi, i, depth))) {
          val phiCopy = phi.map(x=>x)
          phiCopy(depth) = Set[Int](i)
          //if (matches.size >= 1000000) return
          saltzIso_(g, q, saltzDualSim(g, q, phiCopy), depth + 1)
        }
      }
    }

    def contains(phi: Array[Set[Int]], ele: Int, depth: Int): Boolean = {
      for (i <- 0 until depth) if (phi(i) contains ele) return true
      false 
    }
    matches 
  }

  def saltzIsoAltFeasCapped(g: Graph, q: Graph, cap: Int): Set[Array[Set[Int]]] = {
    val start = System.nanoTime()
    var matches = Set[Array[Set[Int]]]()
    saltzIso_(g, q, saltzDualSim(g, q, feasibleMates2(g, q)), 0) 
  
    def saltzIso_(g: Graph, q: Graph, phi: Array[Set[Int]], depth: Int) {
      if (depth == q.size) {
        if (!phi.isEmpty) { 
          matches += phi
        if (matches.size == 1000)
          println("1000 saltzIsoAltFeas: " + (System.nanoTime()-start)/1000000.0)
        if (matches.size % 5000000 == 0)
          println("matches so far: " + matches.size)
        }
      }
      else if (!phi.isEmpty) {
        for (i <- phi(depth) if (!contains(phi, i, depth))) {
          val phiCopy = phi.map(x=>x)
          phiCopy(depth) = Set[Int](i)
          if (matches.size >= cap) return
          saltzIso_(g, q, saltzDualSim(g, q, phiCopy), depth + 1)
        }
      }
    }

    def contains(phi: Array[Set[Int]], ele: Int, depth: Int): Boolean = {
      for (i <- 0 until depth) if (phi(i) contains ele) return true
      false 
    }
    matches 
  }
  def saltzIso3PrintDual(g: Graph, q: Graph): Set[Array[Set[Int]]] = {
    val start = System.nanoTime()
    var matches = Set[Array[Set[Int]]]() 
    val (initPhi, dualTime) = Utils.time{
      saltzDualSim(g, q, feasibleMates(g, q))
    }
    println("dual time: " + dualTime)
    saltzIso_(g, q, initPhi , 0) 
  
    def saltzIso_(g: Graph, q: Graph, phi: Array[Set[Int]], depth: Int) {
      if (depth == q.size) {
        if (!phi.isEmpty) { 
          matches += phi
          if (matches.size == 1000)
            println("1000 saltzIso3: " + (System.nanoTime()-start)/1000000.0)
          if (matches.size % 5000000 == 0)
            println("matches so far: " + matches.size)
        }
      } else if (!phi.isEmpty) {
        for (i <- phi(depth) if (!contains(phi, i, depth))) {
          val phiCopy = phi.map(x=>x)
          phiCopy(depth) = Set[Int](i)
          //if (matches.size >= 1000000) return
          saltzIso_(g, q, saltzDualSim(g, q, phiCopy), depth + 1)
        }
      }
    }

    def contains(phi: Array[Set[Int]], ele: Int, depth: Int): Boolean = {
      for (i <- 0 until depth) if (phi(i) contains ele) return true
      false 
    }
    matches 
  }

  def saltzIso3(g: Graph, q: Graph): Set[Array[Set[Int]]] = {
    val start = System.nanoTime()
    var matches = Set[Array[Set[Int]]]()
    saltzIso_(g, q, saltzDualSim(g, q, feasibleMates(g, q)), 0) 
  
    def saltzIso_(g: Graph, q: Graph, phi: Array[Set[Int]], depth: Int) {
      if (depth == q.size) {
        if (!phi.isEmpty) { 
          matches += phi
          if (matches.size == 1000)
            println("1000 saltzIso3: " + (System.nanoTime()-start)/1000000.0)
          if (matches.size % 5000000 == 0)
            println("matches so far: " + matches.size)
        }
      } else if (!phi.isEmpty) {
        for (i <- phi(depth) if (!contains(phi, i, depth))) {
          val phiCopy = phi.map(x=>x)
          phiCopy(depth) = Set[Int](i)
          //if (matches.size >= 1000000) return
          saltzIso_(g, q, saltzDualSim(g, q, phiCopy), depth + 1)
        }
      }
    }

    def contains(phi: Array[Set[Int]], ele: Int, depth: Int): Boolean = {
      for (i <- 0 until depth) if (phi(i) contains ele) return true
      false 
    }
    matches 
  }

  def saltzIso3Capped(g: Graph, q: Graph, cap: Int): Set[Array[Set[Int]]] = {
    val start = System.nanoTime()
    var matches = Set[Array[Set[Int]]]()
    saltzIso_(g, q, saltzDualSim(g, q, feasibleMates(g, q)), 0) 
  
    def saltzIso_(g: Graph, q: Graph, phi: Array[Set[Int]], depth: Int) {
      if (depth == q.size) {
        if (!phi.isEmpty) { 
          matches += phi
          if (matches.size == 1000)
            println("1000 saltzIso3: " + (System.nanoTime()-start)/1000000.0)
          if (matches.size % 5000000 == 0)
            println("matches so far: " + matches.size)
        }
      } else if (!phi.isEmpty) {
        for (i <- phi(depth) if (!contains(phi, i, depth))) {
          val phiCopy = phi.map(x=>x)
          phiCopy(depth) = Set[Int](i)
          if (matches.size >= cap) return
          saltzIso_(g, q, saltzDualSim(g, q, phiCopy), depth + 1)
        }
      }
    }

    def contains(phi: Array[Set[Int]], ele: Int, depth: Int): Boolean = {
      for (i <- 0 until depth) if (phi(i) contains ele) return true
      false 
    }
    matches 
  }

  def saltzGraphSim(g: Graph, q: Graph): Array[Set[Int]] = 
    saltzGraphSim(g, q, feasibleMates(g, q))

  def saltzGraphSim(g: Graph, q: Graph, phi: Array[Set[Int]]): Array[Set[Int]] = {
    var changed = true
    while (changed) {
      changed = false
      for (qNode <- 0 until q.size) {
        for (qChild <- q.adjList(qNode)) {
          for (phiNode <- phi(qNode)) {
            if ((g.adjList(phiNode) & phi(qChild)).isEmpty) { 
              phi(qNode) -= phiNode 
              if (phi(qNode).isEmpty) return Array[Set[Int]]()
              changed = true  
            } // if
          } // for
        } // for
      } // for
    } // while
    phi
  }
  def saltzDualSimOrdered(g: Graph, q: Graph): Array[Set[Int]] = 
    saltzDualSimOrdered(g, q, feasibleMates(g, q))

  def saltzDualSimOrdered(g: Graph, q: Graph, phi: Array[Set[Int]]): Array[Set[Int]] = {
    var changed = true
    val orderedQ = phi.zipWithIndex.sortBy(x => x._1.size/q.adjList(x._2).size.toDouble).map{ case(list, i) => i }
    while (changed) {
      changed = false
      for (qNode <- orderedQ) {
        for (qChild <- q.adjList(qNode)) {
          // newPhi corresponds to phi(qChild).  This update
          // will ensure that phi(qChild) will contain only 
          // nodes which have a parent in phi(qNode)
          var newPhi = Set[Int]()
          for (phiNode <- phi(qNode)) {
            //println(phiNode)
            //println("nphi " + newPhi)
            //println(phi(qNode))
            //println
            // phiTemp corresponds to the children of 
            // phiNode which are contained in phi(qChild).
            // This checks both if phiNode has children in
            // phi(qChild) (of which it must have at least one)
            // and also builds newPhi to contain only those
            // nodes in phi(qChild) which also have a parent
            // in phi(qNode)
            val phiTemp = g.adjList(phiNode) & phi(qChild)
            if (phiTemp.isEmpty) { 
              phi(qNode) -= phiNode 
              if (phi(qNode).isEmpty) return Array[Set[Int]]()
              changed = true
            //  println("changed with qNode " + qNode + 
            //    "and qChild " + qChild + 
            //    "and phiNode " + phiNode + 
            //    "and phi(qNode) " + phi(qNode))
            }
            
            newPhi = newPhi union phiTemp
          }
          // if any phi(i) is empty, then there is no 
          // isomorphic subgraph.
          if (newPhi.isEmpty) return Array[Set[Int]]() 
          // changed only becomes true if newPhi is smaller than phi(qChild)
          // thus the loop repeats only if at least one phi(n) shrinks
          if (newPhi.size < phi(qChild).size) changed = true
          // every node in phi(qChild) must have at least one parent
          // in phi(qNode)

          if (qChild == qNode) phi(qChild) &= newPhi 
          else phi(qChild) = newPhi
        }
      }
    }
    phi
  }

  def saltzDualSim(g: Graph, q: Graph): Array[Set[Int]] = 
    saltzDualSim(g, q, feasibleMates(g, q))

  def saltzDualSim(g: Graph, q: Graph, phi: Array[Set[Int]]): Array[Set[Int]] = {
    var changed = true
    while (changed) {
      changed = false
      for (qNode <- 0 until q.size) {
        for (qChild <- q.adjList(qNode)) {
          // newPhi corresponds to phi(qChild).  This update
          // will ensure that phi(qChild) will contain only 
          // nodes which have a parent in phi(qNode)
          var newPhi = Set[Int]()
          for (phiNode <- phi(qNode)) {
            //println(phiNode)
            //println("nphi " + newPhi)
            //println(phi(qNode))
            //println
            // phiTemp corresponds to the children of 
            // phiNode which are contained in phi(qChild).
            // This checks both if phiNode has children in
            // phi(qChild) (of which it must have at least one)
            // and also builds newPhi to contain only those
            // nodes in phi(qChild) which also have a parent
            // in phi(qNode)
            val phiTemp = g.adjList(phiNode) & phi(qChild)
            if (phiTemp.isEmpty) { 
              phi(qNode) -= phiNode 
              if (phi(qNode).isEmpty) return Array[Set[Int]]()
              changed = true
            //  println("changed with qNode " + qNode + 
            //    "and qChild " + qChild + 
            //    "and phiNode " + phiNode + 
            //    "and phi(qNode) " + phi(qNode))
            }
            
            newPhi = newPhi union phiTemp
          }
          // if any phi(i) is empty, then there is no 
          // isomorphic subgraph.
          if (newPhi.isEmpty) return Array[Set[Int]]() 
          // changed only becomes true if newPhi is smaller than phi(qChild)
          // thus the loop repeats only if at least one phi(n) shrinks
          if (newPhi.size < phi(qChild).size) changed = true
          // every node in phi(qChild) must have at least one parent
          // in phi(qNode)

          if (qChild == qNode) phi(qChild) &= newPhi 
          else phi(qChild) = newPhi
        }
      }
    }
    phi
  }
 
  def feasibleMates(g: Graph, q: Graph): Array[Set[Int]] = 
    q.labels.map(qNodeLabel => g.getNodesWithLabel(qNodeLabel))

  def feasibleMates2(g: Graph, q: Graph): Array[Set[Int]] = 
    // gives (nodelabel, nodeId) pair
    q.labels.zipWithIndex.map(n => 
      g.getNodesWithLabel(n._1).filter(g.adjList(_).size >= q.adjList(n._2).size)
    )
    

  def writeMatchesToFile(phi: Set[Array[Set[Int]]], fileName: String) {
    val out = new PrintWriter(fileName)
    phi.foreach { p => p.foreach(x => x.foreach(out.println(_))); out.println }
    out.close
  }
}

object LoopTest extends App {
  val qAdjList = Array(Set(0))
  val qLabels = Array(0)
  val qLabelMap = Map((0 -> Set(0)))
  val q = new Graph(qAdjList, qLabels, qLabelMap)
  q.print
  val gAdjList = Array(Set[Int](), Set(1, 0))
  val gLabels = Array(0, 0)
  val gLabelMap = Map((0 -> Set(1, 0)))
  val g = new Graph(gAdjList, gLabels, gLabelMap)
  g.print

  PatternMatcher.saltzDualSim(g, q).foreach ( 
    println(_)
  )
}
