import sys.process._
import scala.actors.Futures._

object Utils {

  def simpleExp(g: Graph, q: Graph, name: String) {
//    println()
//    println("Beginning graph sim...")
    var sum = 0.0 // for average time
    var fSim = Array[Set[Int]]()
    val reps = 5
    (0 to reps).foreach { i => 
      val (sim, time) = Utils.time {
        PatternMatcher.saltzGraphSim(g, q)
      }
      println("time simple: " + time)
      if (i != 0) sum += time
      else fSim = sim
    }
    val uniqueNodes = fSim.flatten.toSet.size
    val avTime = (sum/reps) 
    val minPhi = if (fSim isEmpty) 0
                 else fSim.map(_.size).min
    val maxPhi = if (fSim isEmpty) 0
                 else fSim.map(_.size).max

    println(s"simple,0,$avTime,$uniqueNodes,$minPhi,$maxPhi,$name")
    //println("Simple " + name + " took (av.):   " + (sum/reps) + " ms")
    //println("Simple " + name + " min phi size: " + fSim.map(_.size).min)
    //println("Simple " + name + " max phi size: " + fSim.map(_.size).max)
    //println("Simple " + name + " unique nodes: " + fSim.flatten.toSet.size)
  }

  def orderedDualExp(g: Graph, q: Graph, name: String) {
//    println()
//    println("Beginning dual sim...")
    var sum = 0.0 // for average time
    var fSim = Array[Set[Int]]()
    val reps = 5
    (0 to reps).foreach { i => 
      val (sim, time) = Utils.time {
        PatternMatcher.saltzDualSimOrdered(g, q)
      }
      println("time ordered dual: " + time)
      if (i != 0) sum += time
      else fSim = sim
    }

    val uniqueNodes = fSim.flatten.toSet.size
    val avTime = (sum/reps) 
    val minPhi = if (fSim isEmpty) 0
                 else fSim.map(_.size).min
    val maxPhi = if (fSim isEmpty) 0
                 else fSim.map(_.size).max
    println(s"dual,0,$avTime,$uniqueNodes,$minPhi,$maxPhi,$name")
    //println("Dual " + name + " took (av.):   " + (sum/reps) + " ms")
    //println("Dual " + name + " min phi size: " + fSim.map(_.size).min)
    //println("Dual " + name + " max phi size: " + fSim.map(_.size).max)
    //println("Dual " + name + " unique nodes: " + fSim.flatten.toSet.size)
  }

  def dualExp(g: Graph, q: Graph, name: String) {
//    println()
//    println("Beginning dual sim...")
    var sum = 0.0 // for average time
    var fSim = Array[Set[Int]]()
    val reps = 5
    (0 to reps).foreach { i => 
      val (sim, time) = Utils.time {
        PatternMatcher.saltzDualSim(g, q)
      }
      println("time dual: " + time)
      if (i != 0) sum += time
      else fSim = sim
    }

    val uniqueNodes = fSim.flatten.toSet.size
    val avTime = (sum/reps) 
    val minPhi = if (fSim isEmpty) 0
                 else fSim.map(_.size).min
    val maxPhi = if (fSim isEmpty) 0
                 else fSim.map(_.size).max
    println(s"dual,0,$avTime,$uniqueNodes,$minPhi,$maxPhi,$name")
    //println("Dual " + name + " took (av.):   " + (sum/reps) + " ms")
    //println("Dual " + name + " min phi size: " + fSim.map(_.size).min)
    //println("Dual " + name + " max phi size: " + fSim.map(_.size).max)
    //println("Dual " + name + " unique nodes: " + fSim.flatten.toSet.size)
  }

  def saltzExpCapped(g: Graph, q: Graph, cap: Int, name: String) {
//    println()
//    println("Beginning Saltz Iso...") 
    var sum = 0.0 // for average time
    var fSim = Set[Array[Set[Int]]]()
    var reps = 5
    val repsToCount = 3
    (0 to reps).foreach { i => 
      val (sim, time) = Utils.time {
        runWithTimeout(600000, Set[Array[Set[Int]]]()) {
          PatternMatcher.saltzIso3Capped(g, q, cap)
        }
      }
      println("time saltz: " + time)
      if (i >= repsToCount) sum += time
      if (i == 0) fSim = sim
    }

    val uniqueNodes = fSim.flatten.toSet.size
    val avTime = (sum/repsToCount) 
    val matches = fSim.size

    println(s"saltzIso,$matches,$avTime,$uniqueNodes,0,0,$name")
    //println("Saltz Iso " + name + " took (av.):   " + (sum/reps) + " ms")
    //println("Saltz Iso " + name + " matches:      " + fSim.size)
    //println("Saltz Iso " + name + " unique nodes: " + fSim.flatten.toSet.size)
  }

  def saltzExp(g: Graph, q: Graph, name: String) {
//    println()
//    println("Beginning Saltz Iso...") 
    var sum = 0.0 // for average time
    var fSim = Set[Array[Set[Int]]]()
    var reps = 5
    var repsToCount = 3
    (0 to reps).foreach { i => 
      val (sim, time) = Utils.time {
        runWithTimeout(600000, Set[Array[Set[Int]]]()) {
          PatternMatcher.saltzIso3(g, q)
        }
      }
      println("time saltz: " + time)
      if (i >= repsToCount ) sum += time
      if (i == 0 ) fSim = sim
    }

    val uniqueNodes = fSim.flatten.toSet.size
    val avTime = (sum/repsToCount) 
    val matches = fSim.size

    println(s"saltzIso,$matches,$avTime,$uniqueNodes,0,0,$name")
    //println("Saltz Iso " + name + " took (av.):   " + (sum/reps) + " ms")
    //println("Saltz Iso " + name + " matches:      " + fSim.size)
    //println("Saltz Iso " + name + " unique nodes: " + fSim.flatten.toSet.size)
  }

  def saltzExpWithDualExp(g: Graph, q: Graph, name: String) {
//    println()
//    println("Beginning Saltz Iso...") 
    var sum = 0.0 // for average time
    var fSim = Set[Array[Set[Int]]]()
    var reps = 5
    var repsToCount = 3
    (0 to reps).foreach { i => 
      val (sim, time) = Utils.time {
        runWithTimeout(600000, Set[Array[Set[Int]]]()) {
          PatternMatcher.saltzIso3PrintDual(g, q)
        }
      }
      println("time saltz: " + time)
      if (i >= repsToCount) sum += time
      if (i == 0) fSim = sim
    }

    val uniqueNodes = fSim.flatten.toSet.size
    val avTime = (sum/repsToCount) 
    val matches = fSim.size

    println(s"saltzIso,$matches,$avTime,$uniqueNodes,0,0,$name")
    //println("Saltz Iso " + name + " took (av.):   " + (sum/reps) + " ms")
    //println("Saltz Iso " + name + " matches:      " + fSim.size)
    //println("Saltz Iso " + name + " unique nodes: " + fSim.flatten.toSet.size)
  }

  def saltzAltFeasExpCapped(g: Graph, q: Graph, cap: Int, name: String) {
    //println()
    //println("Beginning Saltz Alt Feas Iso...") 
    var sum = 0.0 // for average time
    var fSim = Set[Array[Set[Int]]]()
    var reps = 5
    val repsToCount = 3
    (0 to reps).foreach { i => 
      val (sim, time) = Utils.time {
        runWithTimeout(600000, Set[Array[Set[Int]]]()) {
          PatternMatcher.saltzIsoAltFeasCapped(g, q, cap)
        }
      }
      println("time saltzAlt: " + time)
      if (i >= repsToCount) sum += time
      if (i == 0) fSim = sim
    }

    val uniqueNodes = fSim.flatten.toSet.size
    val avTime = (sum/repsToCount) 
    val matches = fSim.size

    println(s"saltzAltFeas,$matches,$avTime,$uniqueNodes,0,0,$name")
    //println("Saltz alt feas Iso " + name + " took (av.):   " + (sum/reps) + " ms")
    //println("Saltz alt feas Iso " + name + " matches:      " + fSim.size)
    //println("Saltz alt feas Iso " + name + " unique nodes: " + fSim.flatten.toSet.size)
  }

  def saltzAltFeasExp(g: Graph, q: Graph, name: String) {
    //println()
    //println("Beginning Saltz Alt Feas Iso...") 
    var sum = 0.0 // for average time
    var fSim = Set[Array[Set[Int]]]()
    var reps = 5
    val repsToCount = 3
    (0 to reps).foreach { i => 
      val (sim, time) = Utils.time {
        runWithTimeout(600000, Set[Array[Set[Int]]]()) {
          PatternMatcher.saltzIsoAltFeas(g, q)
        }
      }
      println("time saltzAlt: " + time)
      if (i >= repsToCount) sum += time
      if (i == 0) fSim = sim
    }

    val uniqueNodes = fSim.flatten.toSet.size
    val avTime = (sum/repsToCount) 
    val matches = fSim.size

    println(s"saltzAltFeas,$matches,$avTime,$uniqueNodes,0,0,$name")
    //println("Saltz alt feas Iso " + name + " took (av.):   " + (sum/reps) + " ms")
    //println("Saltz alt feas Iso " + name + " matches:      " + fSim.size)
    //println("Saltz alt feas Iso " + name + " unique nodes: " + fSim.flatten.toSet.size)
  }


  def ullmannExp(g: Graph, q: Graph, name: String) {
   // println()
   // println("Beginning ullmann's...") 
    var sum = 0.0 // for average time
    var fSim = Set[Array[Set[Int]]]()
    var reps = 5
    val repsToCount = 3
    (0 to reps).foreach { i => 
      val (sim, time) = Utils.time {
        runWithTimeout(600000, Set[Array[Set[Int]]]()) {
          PatternMatcher.ullmann(g, q)
        }
      } 
      println("time ull: " + time)
      if (i >= repsToCount) sum += time
      if (i == 0) fSim = sim
    }
    val uniqueNodes = fSim.flatten.toSet.size
    val avTime = (sum/repsToCount) 
    val matches = fSim.size

    println(s"ullmann,$matches,$avTime,$uniqueNodes,0,0,$name")
   // println("Saltz Simple Iso " + name + " took (av.):   " + (sum/reps) + " ms")
   // println("Saltz Simple Iso " + name + " matches:      " + fSim.size)
   // println("Saltz Simple Iso " + name + " unique nodes: " + fSim.flatten.toSet.size)
  }

  def myUllmannExp(g: Graph, q: Graph, name: String) {
    //println()
    //println("Beginning ullmann's...") 
    var sum = 0.0 // for average time
    var fSim = Set[Array[Set[Int]]]()
    var reps = 5
    val repsToCount = 3 
    (0 to reps).foreach { i => 
      val (sim, time) = Utils.time {
        runWithTimeout(600000, Set[Array[Set[Int]]]()) {
          PatternMatcher.ullmann(g, q)
        }
      }
      println("time myUll: " + time)
      if (i >= 3) sum += time
      if (i == 0) fSim = sim
    }

    val uniqueNodes = fSim.flatten.toSet.size
    val avTime = (sum/repsToCount) 
    val matches = fSim.size

    println(s"myUllmann,$matches,$avTime,$uniqueNodes,0,0,$name")
    //println("Saltz Simple Iso " + name + " took (av.):   " + (sum/reps) + " ms")
    //println("Saltz Simple Iso " + name + " matches:      " + fSim.size)
    //println("Saltz Simple Iso " + name + " unique nodes: " + fSim.flatten.toSet.size)
  }

  def vf2Exp(gName: String, qName: String) {
    s"to -15 600 ./iso_exp.sh $gName $qName" #| "scala -cp node_counter/out/ NodeCounter" !
  }
  
//  def newVf2Exp(g: Graph, q: Graph, gName: String, qName: String) {
//    val (gLabelFile, gEdgeFile) = g.iGraphWriteToFiles(name + "g")
//    val (qLabelFile, qEdgeFile) = q.iGraphWriteToFiles(name + "q")
//    val results = s"to -15 1800 ./c/iso ${g.size} ${q.size} ${gEdgeFile} ${qEdgeFile} ${gLabelFile} ${qLabelFile}".!!
//  }

  def vf2ExpVerbose(gName: String, qName: String): String = {
    s"to -15 3600 ./iso_exp.sh $gName $qName".!!
  }

  def runAllFastExps(g: Graph, q: Graph, gFile: String, qFile: String, name: String) {
    Utils.simpleExp(g, q, name)
    Utils.dualExp(g, q, name)
    //Utils.saltzOptExp(g, q, name)
    Utils.saltzExp(g, q, name)
    //Utils.saltzAltFeasExp(g, q, name)
    Utils.vf2Exp(gFile, qFile)
    //Utils.saltzSimpleExp(g, q, name)
  }

  def runAllMyExps(g: Graph, q: Graph, gFile: String, qFile: String, name: String) {
    Utils.dualExp(g, q, name)
    Utils.saltzExp(g, q, name)
    //Utils.saltzAltFeasExp(g, q, name)
  }

  def runAllMyExpsCapped(g: Graph, q: Graph, gFile: String, 
                   qFile: String, cap: Int, name: String) {
    Utils.dualExp(g, q, name)
    Utils.saltzExpCapped(g, q, cap, name)
    //Utils.saltzAltFeasExpCapped(g, q, cap, name)
  }


  def runAllExps(g: Graph, q: Graph, gFile: String, qFile: String, name: String) {
    Utils.simpleExp(g, q, name)
    Utils.dualExp(g, q, name)
    //Utils.saltzOptExp(g, q, name)
    Utils.saltzExp(g, q, name)
    //Utils.saltzAltFeasExp(g, q, name)
    Utils.ullmannExp(g, q, name)
    //Utils.myUllmannExp(g, q, name)
    Utils.vf2Exp(gFile, qFile)
    //Utils.saltzSimpleExp(g, q, name)
  }

/*--------------------------------------------------------------------------*/
  
//  def saltzSimpleOptExp(g: Graph, q: Graph, name: String) {
//    println()
//    println("Beginning Saltz Simple Iso Opt...") 
//    var sum = 0.0 // for average time
//    var fSim = Set[Array[Set[Int]]]()
//    var reps = 5
//    (0 to reps).foreach { i => 
//      val (sim, time) = Utils.time {
//        PatternMatcher.simpleIsoOpt(g, q)
//      }
//      if (i != 0) sum += time
//      else fSim = sim
//    }
//
//    println("Saltz Simple Iso Opt" + name + " took (av.):   " + (sum/reps) + " ms")
//    println("Saltz Simple Iso Opt" + name + " matches:      " + fSim.size)
//    println("Saltz Simple Iso Opt" + name + " unique nodes: " + fSim.flatten.toSet.size)
//  }
//
//  def saltzOptExp(g: Graph, q: Graph, name: String) {
//    println()
//    println("Beginning Saltz Opt Iso...") 
//    var sum = 0.0 // for average time
//    var fSim = Set[Array[Set[Int]]]()
//    var reps = 5
//    (0 to reps).foreach { i => 
//      val (sim, time) = Utils.time {
//        PatternMatcher.saltzIso4(g, q)
//      }
//      if (i != 0) sum += time
//      else fSim = sim
//    }
//
//    println("Saltz Opt Iso " + name + " took (av.):   " + (sum/reps) + " ms")
//    println("Saltz Opt Iso " + name + " matches:      " + fSim.size)
//    println("Saltz Opt Iso " + name + " unique nodes: " + fSim.flatten.toSet.size)
//  }
//
//  def saltzOptDegExp(g: Graph, q: Graph, name: String) {
//    println()
//    println("Beginning Saltz Opt Deg Iso...") 
//    var sum = 0.0 // for average time
//    var fSim = Set[Array[Set[Int]]]()
//    var reps = 5
//    (0 to reps).foreach { i => 
//      val (sim, time) = Utils.time {
//        PatternMatcher.saltzIsoDegreeOpt(g, q)
//      }
//      if (i != 0) sum += time
//      else fSim = sim
//    }
//
//    println("Saltz Deg. Opt Iso " + name + " took (av.):   " + (sum/reps) + " ms")
//    println("Saltz Deg. Opt Iso " + name + " matches:      " + fSim.size)
//    println("Saltz Deg. Opt Iso " + name + " unique nodes: " + fSim.flatten.toSet.size)
//  }


  def time[R](block: => R): (R, Double) = {
    val t0 = System.nanoTime()
    val result = block // call-by-name
    val t1 = System.nanoTime()
    (result, (t1 - t0)/1000000.0)
  }
  def runWithTimeout[T](timeoutMs: Long, default: T)(f: => T) : T = {
    awaitAll(timeoutMs, future(f)).head.asInstanceOf[Option[T]].getOrElse(default)
  }
}

