import sys.process._
import scala.language.postfixOps

/*
 * For each number of labels:
 *    5X: creates a graph
 *      labels it with random dist labels
 *      labels it with pow dist labels
 *      labels it with rand dist labels
 *    for each of those, performs 10 queries
 *
 */
object VaryLabelsSmall extends App {
  val labelRange = 1 to 10
  val nGraphs = 0 until 5
  val queryReps = 0 until 10 
  val querySize = 4
  val queryAvDegree = 2
  val graphSize = 1000
  val graphAvDegree = 4
  val power = 2.1
  val gFile = "lg_small.txt"
  val qFile = "lq_small.txt"
  var q: Graph = _ 

  for (l <- labelRange) { 
    //println("N LABELS: " + l)
    // TODO:  Should I do multiple queries for each data graph?
    for (r <- nGraphs) {
      // RAND LABEL TESTS
      //println(s"BEGINNING RAND LABEL TESTS rep $r FOR $l LABELS")
      //println("Generating rand graph...")
      var g = GraphGenerator.generateRandomConnectedGraph(graphSize, l, graphAvDegree)
      g.writeToFile(gFile)
      //println("Done generating rand graph...")
      for (qRep <- queryReps) {
        val randName = l+"_rand_"+r+"_"+qRep 
        //println(s"Generating query graph $qRep...")
        q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        //println("Done generating query...")
        Utils.runAllExps(g, q, gFile, qFile, randName)
        // END RAND LABEL TESTS
      }
      //println("END RAND LABEL TESTS")
    }
  }
}

object VaryLabelsBoth520 extends App {
  val labelRange = (1 to 20).map(_*5) 
  val nGraphs = 0 until 5
  val queryReps = 0 until 10 
  val querySize = 10 
  val queryAvDegree = 2
  val graphSize = 20000
  val graphAvDegree = 7 
  val gFile = "lg_both520.txt"
  val qFile = "lq_both520.txt"
  var q: Graph = _ 

  for (r <- nGraphs) {
    for (l <- labelRange) { 
      var g = GraphGenerator.generateRandomConnectedGraph(graphSize, l, graphAvDegree)
      g.writeToFile(gFile)
      for (qRep <- queryReps) {
        val randName = l + "_rand_" + r + "_" + qRep 
        q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        Utils.runAllExps(g, q, gFile, qFile, randName)
      }
    }
  }
}
object VaryLabelsBoth5Large extends App {
  val labelRange = (1 to 20).map(_*5) 
  val nGraphs = 0 until 5
  val queryReps = 0 until 10 
  val querySize = 10 
  val queryAvDegree = 2
  val graphSize = 100000
  val graphAvDegree = 10 
  val gFile = "lg_both5106.txt"
  val qFile = "lq_both5106.txt"
  var q: Graph = _ 

  for (r <- nGraphs) {
    for (l <- labelRange) { 
      var g = GraphGenerator.generateRandomConnectedGraph(graphSize, l, graphAvDegree)
      g.writeToFile(gFile)
      for (qRep <- queryReps) {
        val randName = l + "_rand_" + r + "_" + qRep 
        q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        Utils.runAllExps(g, q, gFile, qFile, randName)
      }
    }
  }
}

object VaryLabelsBoth5 extends App {
  val labelRange = (1 to 20).map(_*5) 
  val nGraphs = 0 until 5
  val queryReps = 0 until 10 
  val querySize = 10 
  val queryAvDegree = 2
  val graphSize = 10000
  val graphAvDegree = 6 
  val gFile = "lg_both5.txt"
  val qFile = "lq_both5.txt"
  var q: Graph = _ 

  for (r <- nGraphs) {
    for (l <- labelRange) { 
      var g = GraphGenerator.generateRandomConnectedGraph(graphSize, l, graphAvDegree)
      g.writeToFile(gFile)
      for (qRep <- queryReps) {
        val randName = l + "_rand_" + r + "_" + qRep 
        q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        Utils.runAllExps(g, q, gFile, qFile, randName)
      }
    }
  }
}

object VaryLabelsBoth extends App {
  val labelRange = (1 to 20).map(_*10) 
  val nGraphs = 0 until 5
  val queryReps = 0 until 10 
  val querySize = 10 
  val queryAvDegree = 2
  val graphSize = 10000
  val graphAvDegree = 10 
  val gFile = "lg_both.txt"
  val qFile = "lq_both.txt"
  var q: Graph = _ 

  for (r <- nGraphs) {
    for (l <- labelRange) { 
      var g = GraphGenerator.generateRandomConnectedGraph(graphSize, l, graphAvDegree)
      g.writeToFile(gFile)
      for (qRep <- queryReps) {
        val randName = l+"_rand_"+r+"_"+qRep 
        q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        Utils.runAllExps(g, q, gFile, qFile, randName)
      }
    }
  }
}

object VaryLabelsSmallPow extends App {
  val labelRange = 1 to 10
  val nGraphs = 0 until 5
  val queryReps = 0 until 10 
  val querySize = 4
  val queryAvDegree = 2
  val graphSize = 1000
  val graphAvDegree = 4
  val power = 2.1
  val gFile = "lg_small_pow.txt"
  val qFile = "lq_small_pow.txt"
  var q: Graph = _ 

  for (l <- labelRange) { 
    //println("N LABELS: " + l)
    // TODO:  Should I do multiple queries for each data graph?
    for (r <- nGraphs) {

      // BEGIN POWER LABEL TESTS
      //println(s"BEGINNING POW LABEL TESTS rep $r FOR $l LABELS")
      //println("Creating graph pow...")
      var g = GraphGenerator.generateRandomConnectedGraph(graphSize, l, graphAvDegree)
      val powLabels = GraphGenerator.powDistLabels(g.size, l, power)
      g = new Graph(g.adjList, powLabels, 
                    Graph.buildLabelMapFromLabels(powLabels))
      g.writeToFile(gFile)
      //println("Done creating graph pow...")

      for (qRep <- queryReps) {
        val powName = l+"_pow_"+r+"_"+qRep 
        //println(s"Creating query graph $qRep...")
        q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        //println("Done creating query graph...")

        Utils.runAllExps(g, q, gFile, qFile, powName)
        //END POWER LABEL TESTS
      }
      //println("END POWER LABEL TESTS")

//      //println(s"BEGINNING GAUSS LABEL TESTS rep $r FOR $l LABELS")
//      //BEGIN GAUSS LABEL TESTS
//      val gaussLabels = GraphGenerator.gaussianDistLabels(g.size, l)
//      g = new Graph(g.adjList, gaussLabels, 
//                    Graph.buildLabelMapFromLabels(gaussLabels))
//      g.writeToFile(gFile)
//      for (qRep <- queryReps) {
//        val gaussName = l+"_gauss_"+r+"_"+qRep  
//        //println(s"generating query graph $qRep")
//        q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
//        q.writeToFile(qFile)
//
//        Utils.runAllExps(g, q, gFile, qFile, gaussName)
//        // END GAUSS LABEL TESTS
//      }
//      //println("END GAUSS LABEL TESTS")
    }
  }
  println("FINISHED EXPERIMENT SUCCESSFULLY THANK GOD!")
}

object VaryLabelsLargePow extends App {
  val labelRange = Array(1000000, 500, 400, 300, 200, 100, 50) 
  val nGraphs = 0 until 5
  val queryReps = 0 until 20 
  val querySize = 10 
  val queryAvDegree = 2
  val graphSize = 1000000
  val graphAvDegree = 16 
  val power = 2.1
  val gFile = "lg_large_pow.txt"
  val qFile = "lq_large_pow.txt"
  var q: Graph = _

  for (l <- labelRange) { 
    //println("N LABELS: " + l)
    for (r <- nGraphs) {
      //println(s"BEGINNING RAND LABEL TESTS rep $r FOR $l LABELS")
      // RAND LABEL TESTS
      //println("Generating rand graph...")
      var g = GraphGenerator.generateRandomConnectedGraph(graphSize, l, graphAvDegree)

      //println(s"BEGINNING POW LABEL TESTS rep $r FOR $l LABELS")
      // BEGIN POWER LABEL TESTS
      //println("Creating graph pow...")
      val powLabels = GraphGenerator.powDistLabels(g.size, l, power)
      g = new Graph(g.adjList, powLabels, 
                    Graph.buildLabelMapFromLabels(powLabels))
      g.writeToFile(gFile)
    //println("Done creating graph pow...")

      for (qRep <- queryReps) {
        val powName = l+"_pow_"+r+"_"+qRep
        //println(s"Creating query graph $qRep...")
        q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        //println("Done creating query graph...")

        Utils.runAllExps(g, q, gFile, qFile, powName)
        //END POWER LABEL TESTS
      }
      //println("END POWER LABEL TESTS")

 //     //println(s"BEGINNING GAUSS LABEL TESTS rep $r FOR $l LABELS")
 //     //BEGIN GAUSS LABEL TESTS
 //     val gaussLabels = GraphGenerator.gaussianDistLabels(g.size, l)
 //     g = new Graph(g.adjList, gaussLabels, 
 //                   Graph.buildLabelMapFromLabels(gaussLabels))
 //     g.writeToFile(gFile)
 //     for (qRep <- queryReps) {
 //       val gaussName = l+"_gauss_"+r+"_"+qRep
 //       //println(s"generating query graph $qRep")
 //       q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
 //       q.writeToFile(qFile)

 //       Utils.runAllExps(g, q, gFile, qFile, gaussName)
        // END GAUSS LABEL TESTS
     // }
    }
  }
  println("SUCCESSFULLY FINISHED THANK THE LORD ALMIGHTY!")
}

object VaryLabelsLarge extends App {
  val labelRange = Array(1000000, 500, 400, 300, 200, 100, 50) 
  val nGraphs = 0 until 5
  val queryReps = 0 until 20 
  val querySize = 10 
  val queryAvDegree = 2
  val graphSize = 1000000
  val graphAvDegree = 16 
  val power = 2.1
  val gFile = "lg_large.txt"
  val qFile = "lq_large.txt"
  var q: Graph = _

  for (l <- labelRange) { 
    //println("N LABELS: " + l)
    for (r <- nGraphs) {
      //println(s"BEGINNING RAND LABEL TESTS rep $r FOR $l LABELS")
      // RAND LABEL TESTS
      //println("Generating rand graph...")
      var g = GraphGenerator.generateRandomConnectedGraph(graphSize, l, graphAvDegree)
      g.writeToFile(gFile)
      //println("Done generating rand graph...")

      for (qRep <- queryReps) {
        val randName =  l+"_rand_"+r+"_"+qRep 
        //println(s"Generating query graph $qRep...")
        q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        //println("Done generating query...")

        Utils.runAllExps(g, q, gFile, qFile, randName)
        // END RAND LABEL TESTS
      }
      //println("END RAND LABEL TESTS")

      //println(s"BEGINNING POW LABEL TESTS rep $r FOR $l LABELS")
      // BEGIN POWER LABEL TESTS
      //println("Creating graph pow...")
 //     val powLabels = GraphGenerator.powDistLabels(g.size, l, power)
 //     g = new Graph(g.adjList, powLabels, 
 //                   Graph.buildLabelMapFromLabels(powLabels))
 //     g.writeToFile(gFile)
      //println("Done creating graph pow...")

 //     for (qRep <- queryReps) {
 //       val powName = l+"_pow_"+r+"_"+qRep
 //       //println(s"Creating query graph $qRep...")
 //       q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
 //       q.writeToFile(qFile)
 //       //println("Done creating query graph...")

 //       Utils.runAllExps(g, q, gFile, qFile, powName)
 //       //END POWER LABEL TESTS
 //     }
 //     //println("END POWER LABEL TESTS")

 //     //println(s"BEGINNING GAUSS LABEL TESTS rep $r FOR $l LABELS")
 //     //BEGIN GAUSS LABEL TESTS
 //     val gaussLabels = GraphGenerator.gaussianDistLabels(g.size, l)
 //     g = new Graph(g.adjList, gaussLabels, 
 //                   Graph.buildLabelMapFromLabels(gaussLabels))
 //     g.writeToFile(gFile)
 //     for (qRep <- queryReps) {
 //       val gaussName = l+"_gauss_"+r+"_"+qRep
 //       //println(s"generating query graph $qRep")
 //       q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
 //       q.writeToFile(qFile)

 //       Utils.runAllExps(g, q, gFile, qFile, gaussName)
        // END GAUSS LABEL TESTS
     // }
    }
  }
  println("SUCCESSFULLY FINISHED THANK THE LORD ALMIGHTY!")
}
