import sys.process._
import scala.language.postfixOps

object VaryVgRandDegreeLarge extends App {
  val l = 100
  val nQueries = 0 until 10
  val nGraphs = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = Array(1000000, 2000000, 3000000, 4000000, 5000000, 
                         6000000, 7000000, 8000000, 9000000, 10000000)
  val graphAvDegrees = graphSizes.map(math.pow(_, 0.2).toInt) 
  val graphExps = graphSizes zip graphAvDegrees
  val gFile = "vgrandlarge.txt"
  val qFile = "vqrandlarge.txt"

  for (r <- nGraphs) {
    for ((vg, deg) <- graphExps) {
      var g = GraphGenerator.generateRandomConnectedGraph(vg, l, deg)
      g.writeToFile(gFile)

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+r+"_"+qRep
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        Utils.runAllExps(g, q, gFile, qFile, name)
      }
    }
  }
}

object VaryVgRandDegreeLargeNoUll extends App {
  val l = 100
  val nQueries = 0 until 10
  val nGraphs = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = Array(1000000, 2000000, 3000000, 4000000, 5000000, 
                         6000000, 7000000, 8000000, 9000000, 10000000)
  val graphAvDegrees = graphSizes.map(math.pow(_, 0.2).toInt) 
  val graphExps = graphSizes zip graphAvDegrees
  val gFile = "vgrandlargefast.txt"
  val qFile = "vqrandlargefast.txt"

  for (r <- nGraphs) {
    for ((vg, deg) <- graphExps) {
      var g = GraphGenerator.generateRandomConnectedGraph(vg, l, deg)
      g.writeToFile(gFile)

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+r+"_"+qRep
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        Utils.runAllFastExps(g, q, gFile, qFile, name)
      }
    }
  }
}
object VaryVgRandDegreeLargeJustUll4 extends App {
  val l = 100
  val nQueries = 0 until 10
  val nGraphs = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = Array(4000000)
  val graphAvDegrees = graphSizes.map(math.pow(_, 0.2).toInt) 
  val graphExps = graphSizes zip graphAvDegrees
  val gFile = "vgrandlargejur4.txt"
  val qFile = "vqrandlargejur4.txt"

  for (r <- nGraphs) {
    for ((vg, deg) <- graphExps) {
      var g = GraphGenerator.generateRandomConnectedGraph(vg, l, deg)

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+r+"_"+qRep
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        Utils.ullmannExp(g, q, name) 
      }
    }
  }
}

object VaryVgRandDegreeLargeJustUll5 extends App {
  val l = 100
  val nQueries = 0 until 10
  val nGraphs = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = Array(5000000)
  val graphAvDegrees = graphSizes.map(math.pow(_, 0.2).toInt) 
  val graphExps = graphSizes zip graphAvDegrees
  val gFile = "vgrandlargejur5.txt"
  val qFile = "vqrandlargejur5.txt"

  for (r <- nGraphs) {
    for ((vg, deg) <- graphExps) {
      var g = GraphGenerator.generateRandomConnectedGraph(vg, l, deg)

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+r+"_"+qRep
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        Utils.ullmannExp(g, q, name) 
      }
    }
  }
}
object VaryVgRandDegreeLargeJustUll extends App {
  val l = 100
  val nQueries = 0 until 10
  val nGraphs = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = (1 to 10).map(_*1000000)
  val graphAvDegrees = graphSizes.map(math.pow(_, 0.2).toInt) 
  val graphExps = graphSizes zip graphAvDegrees
  val gFile = "vgrandlargeju.txt"
  val qFile = "vqrandlargeju.txt"

  for (r <- nGraphs) {
    for ((vg, deg) <- graphExps) {
      var g = GraphGenerator.generateRandomConnectedGraph(vg, l, deg)

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+r+"_"+qRep
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        Utils.ullmannExp(g, q, name) 
      }
    }
  }
}


object VaryVgRandDegreeLargeMe extends App {
  val l = 100
  val nQueries = 0 until 10
  val nGraphs = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = Array(1000000, 2000000, 3000000, 4000000, 5000000, 
                         6000000, 7000000, 8000000, 9000000, 10000000)
  val graphAvDegrees = graphSizes.map(math.pow(_, 0.2).toInt) 
  val graphExps = graphSizes zip graphAvDegrees
  val gFile = "vgrandlargeme_.txt"
  val qFile = "vqrandlargeme_.txt"

  for (r <- nGraphs) {
    for ((vg, deg) <- graphExps) {
      var g = GraphGenerator.generateRandomConnectedGraph(vg, l, deg)
      g.writeToFile(gFile)

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+r+"_"+qRep
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        if (vg > 3000000)
          Utils.runAllMyExps(g, q, gFile, qFile, name)
        else Utils.runAllFastExps(g, q, gFile, qFile, name) 
      }
    }
  }
}

object VaryVgPowDegreeLargeJustUll extends App {
  val l = 100
  val nQueries = 0 until 10
  val nGraphs = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = (1 to 10).map(_*1000000)
  val graphMaxDegrees = graphSizes.map(x=>(math.pow(x, 0.2)*4.41).toInt) 
  val graphExps = graphSizes zip graphMaxDegrees
  val pow = 2.1
  val gFile = "vgpowlargeju.txt"
  val qFile = "vqpowlargeju.txt"

  for (r <- nGraphs) {
    for ((vg, deg) <- graphExps) {
      var g = GraphGenerator.generatePLGraphRandLabels(vg, l, deg, pow)

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+r+"_"+qRep
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        Utils.ullmannExp(g, q, name)
      }
    }
  }
}

object VaryVgPowDegreeLargeJustUll5 extends App {
  val l = 100
  val nQueries = 0 until 10
  val nGraphs = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = Array(5000000)
  val graphMaxDegrees = graphSizes.map(x=>(math.pow(x, 0.2)*4.41).toInt) 
  val graphExps = graphSizes zip graphMaxDegrees
  val pow = 2.1
  val gFile = "vgpowlargeju5.txt"
  val qFile = "vqpowlargeju5.txt"

  for (r <- nGraphs) {
    for ((vg, deg) <- graphExps) {
      var g = GraphGenerator.generatePLGraphRandLabels(vg, l, deg, pow)

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+r+"_"+qRep
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        Utils.ullmannExp(g, q, name)
      }
    }
  }
}
object VaryVgPowDegreeLargeJustUll4 extends App {
  val l = 100
  val nQueries = 0 until 10
  val nGraphs = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = Array(4000000)
  val graphMaxDegrees = graphSizes.map(x=>(math.pow(x, 0.2)*4.41).toInt) 
  val graphExps = graphSizes zip graphMaxDegrees
  val pow = 2.1
  val gFile = "vgpowlargeju4.txt"
  val qFile = "vqpowlargeju4.txt"

  for (r <- nGraphs) {
    for ((vg, deg) <- graphExps) {
      var g = GraphGenerator.generatePLGraphRandLabels(vg, l, deg, pow)

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+r+"_"+qRep
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        Utils.ullmannExp(g, q, name)
      }
    }
  }
}




object VaryVgPowDegreeLarge extends App {
  val l = 100
  val nQueries = 0 until 10
  val nGraphs = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = Array(1000000, 2000000, 3000000, 4000000, 5000000,
                         6000000, 7000000, 8000000, 9000000, 10000000)
  val graphMaxDegrees = graphSizes.map(x=>(math.pow(x, 0.2)*4.41).toInt) 
  val graphExps = graphSizes zip graphMaxDegrees
  val pow = 2.1
  val gFile = "vgpowlarge.txt"
  val qFile = "vqpowlarge.txt"

  for (r <- nGraphs) {
    for ((vg, deg) <- graphExps) {
      var g = GraphGenerator.generatePLGraphRandLabels(vg, l, deg, pow)
      g.writeToFile(gFile)

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+r+"_"+qRep
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        Utils.runAllExps(g, q, gFile, qFile, name)
      }
    }
  }
}

object VaryVgPowDegreeLargeNoUll extends App {
  val l = 100
  val nQueries = 0 until 10
  val nGraphs = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = Array(1000000, 2000000, 3000000, 4000000, 5000000,
                         6000000, 7000000, 8000000, 9000000, 10000000)
  val graphMaxDegrees = graphSizes.map(x=>(math.pow(x, 0.2)*4.41).toInt) 
  val graphExps = graphSizes zip graphMaxDegrees
  val pow = 2.1
  val gFile = "vgpowlargefast.txt"
  val qFile = "vqpowlargefast.txt"

  for (r <- nGraphs) {
    for ((vg, deg) <- graphExps) {
      var g = GraphGenerator.generatePLGraphRandLabels(vg, l, deg, pow)
      g.writeToFile(gFile)

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+r+"_"+qRep
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        Utils.runAllFastExps(g, q, gFile, qFile, name)
      }
    }
  }
}

object VaryVgPowDegreeLargeMe extends App {
  val l = 100
  val nQueries = 0 until 10
  val nGraphs = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = Array(1000000, 2000000, 3000000, 4000000, 5000000,
                         6000000, 7000000, 8000000, 9000000, 10000000)
  val graphMaxDegrees = graphSizes.map(x=>(math.pow(x, 0.2)*4.41).toInt) 
  val graphExps = graphSizes zip graphMaxDegrees
  val pow = 2.1
  val gFile = "vgpowlargeme.txt"
  val qFile = "vqpowlargeme.txt"

  for (r <- nGraphs) {
    for ((vg, deg) <- graphExps) {
      var g = GraphGenerator.generatePLGraphRandLabels(vg, l, deg, pow)
      g.writeToFile(gFile)

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+r+"_"+qRep
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        if (vg > 3000000)
          Utils.runAllMyExps(g, q, gFile, qFile, name)
        else Utils.runAllFastExps(g, q, gFile, qFile, name)
      }
    }
  }
}


object VaryVgRandDegree extends App {
  val l = 100
  val nQueries = 0 until 10
  val nGraphs = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = Array(1000, 10000, 100000, 1000000, 10000000)
  val graphAvDegrees = graphSizes.map(math.pow(_, 0.2).toInt) 
  val graphExps = graphSizes zip graphAvDegrees
  val gFile = "vgrand.txt"
  val qFile = "vqrand.txt"

  for ((vg, deg) <- graphExps) {
    //println(s"GRAPH SIZE: $vg   DEGREE: $deg")
    for (r <- nGraphs) {
      //println(s"Outer rep $o\n")
      //println("Generating rand graph...")
      var g = GraphGenerator.generateRandomConnectedGraph(vg, l, deg)
      g.writeToFile(gFile)
      //println("Done generating rand graph...")

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+r+"_"+qRep
        //println("Generating query graph...")
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        //println("Done generating query...")

        if (vg != 10000000) Utils.runAllExps(g, q, gFile, qFile, name)
        else Utils.runAllFastExps(g, q, gFile, qFile, name)
      }
    }
  }
}

object VaryVgPowDegree extends App {
  val l = 100
  val nQueries = 0 until 10
  val nGraphs = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = Array(1000, 10000, 100000, 1000000, 10000000)
  val graphMaxDegrees = graphSizes.map(x=>(math.pow(x, 0.2)*4.41).toInt) 
  val graphExps = graphSizes zip graphMaxDegrees
  val pow = 2.1
  val gFile = "vgpow.txt"
  val qFile = "vqpow.txt"

  for ((vg, deg) <- graphExps) {
    //println(s"GRAPH SIZE: $vg   DEGREE: $deg")
    for (r <- nGraphs) {
      //println(s"Outer rep $o\n")
      //println("Generating pow graph...")
      var g = GraphGenerator.generatePLGraphRandLabels(vg, l, deg, pow)
      g.writeToFile(gFile)
      //println("Done generating pow graph...")

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+r+"_"+qRep
        //println("Generating query graph...")
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        //println("Done generating query...")
        
        if(vg != 10000000) Utils.runAllExps(g, q, gFile, qFile, name)
        else Utils.runAllFastExps(g, q, gFile, qFile, name)
      }
    }
  }
}

object VaryVgRandDegreePropLabels extends App {
  val nGraphs = 0 until 10
  val nQueries = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = Array(1000, 10000, 100000, 1000000, 10000000)
  val graphAvDegrees = graphSizes.map(math.pow(_, 0.2).toInt) 
  val graphExps = graphSizes zip graphAvDegrees
  val gFile = "vgrandprop.txt"
  val qFile = "vqrandprop.txt"

  for ((vg, deg) <- graphExps) {
    val l = math.max(vg/1000, 1)
    //println(s"GRAPH SIZE: $vg   DEGREE: $deg")
    for (r <- nGraphs) {
      //println(s"Outer rep $o\n")
      //println("Generating rand graph...")
      var g = GraphGenerator.generateRandomConnectedGraph(vg, l, deg)
      g.writeToFile(gFile)
      //println("Done generating rand graph...")

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+l+"_"+r+"_"+qRep
        //println("Generating query graph...")
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        //println("Done generating query...")

        Utils.runAllExps(g, q, gFile, qFile, name)
      }
    }
  }
}

object VaryVgPowDegreePropLabels extends App {
  val nGraphs = 0 until 10
  val nQueries = 0 until 5
  val querySize = 10
  val queryAvDegree = 2
  val graphSizes = Array(1000, 10000, 100000, 1000000, 10000000)
  val graphMaxDegrees = graphSizes.map(x=>(math.pow(x, 0.2)*4.41).toInt) 
  val graphExps = graphSizes zip graphMaxDegrees
  val pow = 2.1
  val gFile = "vgpowprop.txt"
  val qFile = "vqpowprop.txt"

  for ((vg, deg) <- graphExps) {
    val l = math.max(vg/1000, 1)
    //println(s"GRAPH SIZE: $vg   DEGREE: $deg")
    for (r <- nGraphs) {
      //println(s"Outer rep $o\n")
      //println("Generating pow graph...")
      var g = GraphGenerator.generatePLGraphRandLabels(vg, l, deg, pow)
      g.writeToFile(gFile)
      //println("Done generating pow graph...")

      for (qRep <- nQueries) {
        val name = vg+"_"+deg+"_"+l+"_"+r+"_"+qRep
        //println("Generating query graph...")
        var q = GraphGenerator.generateBFSQuery(querySize, queryAvDegree, g)
        q.writeToFile(qFile)
        //println("Done generating query...")

        Utils.runAllExps(g, q, gFile, qFile, name)
      }
    }
  }
}
