import scala.io._

object DiffExp extends App {

  val g = GraphGenerator.generatePLGraphPowLabels(1000, 13, 24, 2.1)
  val gFile = "diffg.txt"
  g.writeToFile(gFile)
  val qFile = "diffq.txt"

  val smallQueries = Array((2, 1), (4, 2))
 
  // BFS queries
  for (i <- 0 until 10000) {
    for ((s, deg) <- smallQueries) {
      val name = s + "_" + deg + "_bfs_" + i
      //val q = GraphGenerator.generateBFSQuery(s, deg, g)
      //TODO: fix g.nLabels to return the actual range and not just no.
      //unique
      val q = GraphGenerator.generatePLGraphRandLabels(s, g.nLabels, deg * 4, 2.1) 
      if (q.nSelfLoops > 0) println(s"q has self loops: " + q.nSelfLoops)
      q.writeToFile(qFile)
      //Utils.runAllFastExps(g, q, gFile, qFile, name)
    }
  }
}

object YoutubeBFSExp extends App {

  val gFile = args(0)
  val g = Graph(gFile)
  val qFile = "youqsmallbfs_"+gFile+".txt"
  println("Youtube size: " + g.size)
  println("Youtube edges: " + g.nEdges)
  println("all query sizes, 100 iterations of each, all exps")
  val smallQueries = Array((2, 1), (4, 2), (6, 2), (8, 2), (10, 2))
 
  // BFS queries
  for (i <- 0 until 100) {
    for ((s, deg) <- smallQueries) {
      val name = s + "_" + deg + "_bfs_" + i
      val q = GraphGenerator.generateBFSQuery(s, deg, g)
      q.writeToFile(qFile)
      Utils.runAllFastExps(g, q, gFile, qFile, name)
    }
  }
}

object WikiBFSExp extends App {

  val gFile = args(0)
  val g = Graph(gFile)
  val qFile = "wikiqsmallbfs_"+gFile+".txt"
  println("Wiki size: " + g.size)
  println("Wiki edges: " + g.nEdges)
  println("all query sizes, 100 iterations of each, all exps")
  val smallQueries = Array((2, 1), (4, 2), (6, 2), (8, 2), (10, 2))
 
  // BFS queries
  for (i <- 0 until 100) {
    for ((s, deg) <- smallQueries) {
      val name = s + "_" + deg + "_bfs_" + i
      val q = GraphGenerator.generateBFSQuery(s, deg, g)
      q.writeToFile(qFile)
      Utils.runAllFastExps(g, q, gFile, qFile, name)
    }
  }
}

object WikiRandAllExp extends App {
  val gFile = args(0)
  val g = Graph(gFile) 
  val qFile = "wikiqsmallrand_"+gFile+".txt"
  println("Wiki size: " + g.size)
  println("Wiki nEdges: " + g.nEdges)
  println("all query sizes, 100 iterations of each, all exps")

  val smallQueries = Array((2, 1), (4, 2), (6, 2), (8, 2), (10, 2))

  // Rand queries
  for (i <- 0 until 100) {
    for ((s, deg) <- smallQueries) {
      val name = s + "_" + deg + "_rand_" + i
      val q = GraphGenerator.generateRandomConnectedGraph(s, g.nLabels, deg) 
      q.writeToFile(qFile)
      println("Done generating query graph")
      Utils.runAllFastExps(g, q, gFile, qFile, name)
    }
  }
}


object YoutubeBigBFSExp extends App {

  val gFile = args(0)
  val g = Graph(gFile) 
  val qFile = "youqbigbfs_"+gFile+".txt"

  val bigQueries = Array((6, 2), (8, 2), (10, 2))
 
  // BFS queries
  for (i <- 0 until 100) {
    for ((s, deg) <- bigQueries) {
      val name = s + "_" + deg + "_bfs_" + i
      val q = GraphGenerator.generateBFSQuery(s, deg, g)
      q.writeToFile(qFile)
      Utils.runAllFastExps(g, q, gFile, qFile, name)
    }
  }
}


object YoutubeRandAllExp extends App {
  val gFile = args(0)
  val g = Graph(gFile) 
  val qFile = "youqsmallrand_"+gFile+".txt"
  println("Youtube size: " + g.size)
  println("Youtube nEdges: " + g.nEdges)
  println("all query sizes, 100 iterations of each, all exps")

  val smallQueries = Array((2, 1), (4, 2), (6, 2), (8, 2), (10, 2))

  // Rand queries
  for (i <- 0 until 100) {
    for ((s, deg) <- smallQueries) {
      val name = s + "_" + deg + "_rand_" + i
      val q = GraphGenerator.generateRandomConnectedGraph(s, g.nLabels, deg) 
      q.writeToFile(qFile)
      println("Done generating query graph")
      Utils.runAllFastExps(g, q, gFile, qFile, name)
    }
  }
}

object YoutubeRandMyExpCapped extends App {
  val gFile = args(0)
  val g = Graph(gFile) 
  val cap = 1000000
  val qFile = "youqsmallrand_"+gFile+".txt"
  println("Youtube size: " + g.size)
  println("Youtube nEdges: " + g.nEdges)
  println("all query sizes, 100 iterations of each, all exps")

  val smallQueries = Array((2, 1), (4, 2), (6, 2), (8, 2), (10, 2))

  // Rand queries
  for (i <- 0 until 100) {
    for ((s, deg) <- smallQueries) {
      val name = s + "_" + deg + "_rand_" + i
      val q = GraphGenerator.generateRandomConnectedGraph(s, g.nLabels, deg) 
      q.writeToFile(qFile)
      println("Done generating query graph")
      Utils.runAllMyExpsCapped(g, q, gFile, qFile, cap, name)
    }
  }
}


object YoutubeBigRandExp extends App {
  val gFile = args(0)
  val g = Graph(gFile)
  val qFile = "youqbigrand_"+gFile+".txt"

  val bigQueries = Array((6, 2), (8, 2), (10, 2))

  // Rand queries
  for (i <- 0 until 100) {
    for ((s, deg) <- bigQueries) {
      val name = s + "_" + deg + "_rand_" + i
      val q = GraphGenerator.generateRandomConnectedGraph(s, g.nLabels, deg) 
      q.writeToFile(qFile)
      Utils.runAllFastExps(g, q, gFile, qFile, name)
    }
  }
}

object YouConverter extends App{
  readYoutubeGraph(args(0), args(1))
  def readYoutubeGraph (inFile: String, outFile: String) = {
    val lines = Source.fromFile(inFile).getLines.toArray
    var idMap = Map[String, Int]()
    var labelStr = Vector[String]()
    var labelSet = Set[String]()
    for (l <- lines) {
      if (!l.trim.equals("")) {
        val splitLine = l.split("\t").map(_.trim)
        if (splitLine.length >= 9) {
          val label = splitLine(3) 
          labelStr :+= label
          labelSet += label 
          idMap += (splitLine(0) -> idMap.size)
        }
      }
    }
    println(labelSet)
    val labelArr = labelSet.toArray
    val labels = labelStr.toArray.map(x => labelArr.indexOf(x))
    val labelMap = Graph.buildLabelMapFromLabels(labels) 
    val adjList = lines.foldLeft(Vector[Set[Int]]()) { (vec, x) =>
      val sx = x.split("\t").map(_.trim)
      if (sx.length > 9) {
        vec :+ sx.slice(10, sx.length).foldLeft(Set[Int]()){ (set, x) =>
          if(idMap.contains(x)) set + idMap(x) 
          else set 
        }
      } else if (sx.length == 9) {
        vec :+ Set[Int]() 
      } else { vec }
    }.toArray
    val g = new Graph(adjList, labels, labelMap)
    g.writeToFile(outFile)
  }
}

